aboutsummaryrefslogtreecommitdiff
path: root/cores/arduino/wiring_pulse.c
diff options
context:
space:
mode:
Diffstat (limited to 'cores/arduino/wiring_pulse.c')
-rwxr-xr-xcores/arduino/wiring_pulse.c20
1 files changed, 13 insertions, 7 deletions
diff --git a/cores/arduino/wiring_pulse.c b/cores/arduino/wiring_pulse.c
index 642a5a2..0157114 100755
--- a/cores/arduino/wiring_pulse.c
+++ b/cores/arduino/wiring_pulse.c
@@ -26,10 +26,10 @@
#include "pins_arduino.h"
/* Measures the length (in microseconds) of a pulse on the pin; state is HIGH
- * or LOW, the type of pulse to measure. Works on pulses from 10 microseconds
- * to 3 minutes in length, but must be called at least N microseconds before
- * the start of the pulse. */
-unsigned long pulseIn(uint8_t pin, uint8_t state)
+ * or LOW, the type of pulse to measure. Works on pulses from 2-3 microseconds
+ * to 3 minutes in length, but must be called at least a few dozen microseconds
+ * before the start of the pulse. */
+unsigned long pulseIn(uint8_t pin, uint8_t state, unsigned long timeout)
{
// cache the port and bit of the pin in order to speed up the
// pulse width measuring loop and achieve finer resolution. calling
@@ -39,17 +39,23 @@ unsigned long pulseIn(uint8_t pin, uint8_t state)
uint8_t stateMask = (state ? bit : 0);
unsigned long width = 0; // keep initialization out of time critical area
+ // convert the timeout from microseconds to a number of times through
+ // the initial loop; it takes 16 clock cycles per iteration.
+ unsigned long numloops = 0;
+ unsigned long maxloops = microsecondsToClockCycles(timeout) / 16;
+
// wait for the pulse to start
while ((*portInputRegister(port) & bit) != stateMask)
- ;
+ if (numloops++ == maxloops)
+ return 0;
// wait for the pulse to stop
while ((*portInputRegister(port) & bit) == stateMask)
width++;
// convert the reading to microseconds. The loop has been determined
// to be 10 clock cycles long and have about 12 clocks between the edge
// and the start of the loop. There will be some error introduced by
// the interrupt handlers.
- return clockCyclesToMicroseconds(width * 10 + 12);
+ return clockCyclesToMicroseconds(width * 10 + 16);
}
@@ -39,17 +39,23 @@ unsigned long pulseIn(uint8_t pin, uint8_t state)
width++;
// convert the reading to microseconds. The loop has been determined
- // to be 10 clock cycles long and have about 12 clocks between the edge
+ // to be 10 clock cycles long and have about 16 clocks between the edge
// and the start of the loop. There will be some error introduced by
// the interrupt handlers.